@@ -3,35 +3,8 @@
 # Univention Directory Replication
 #  listener module for Directory replication
 #
-# Like what you see? Join us!
-# https://www.univention.com/about-us/careers/vacancies/
-#
-# Copyright 2004-2024 Univention GmbH
-#
-# https://www.univention.de/
-#
-# All rights reserved.
-#
-# The source code of this program is made available
-# under the terms of the GNU Affero General Public License version 3
-# (GNU AGPL V3) as published by the Free Software Foundation.
-#
-# Binary versions of this program provided by Univention to you as
-# well as other copyrighted, protected or trademarked materials like
-# Logos, graphics, fonts, specific documentations and configurations,
-# cryptographic keys etc. are subject to a license agreement between
-# you and Univention and not subject to the GNU AGPL V3.
-#
-# In the case you use this program under the terms of the GNU AGPL V3,
-# the program is provided in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU Affero General Public License for more details.
-#
-# You should have received a copy of the GNU Affero General Public
-# License with the Debian GNU/Linux or Univention distribution in file
-# /usr/share/common-licenses/AGPL-3; if not, see
-# <https://www.gnu.org/licenses/>.
+# SPDX-FileCopyrightText: 2004-2025 Univention GmbH
+# SPDX-License-Identifier: AGPL-3.0-only
 # Univention Directory Listener replication module
 
 # Possible initialization scenarios:
@@ -51,7 +24,7 @@ import sys
 import time
 from email.mime.text import MIMEText
 from errno import ENOENT
-from typing import Any
+from typing import IO, Any
 
 import ldap
 import ldap.schema
@@ -74,10 +47,11 @@ slave = listener.configRegistry['ldap/server/type'] == 'slave'
 if listener.configRegistry['ldap/slave/filter']:
     filter = listener.configRegistry['ldap/slave/filter']
 
-LDAP_DIR = '/var/lib/univention-ldap/'
+LDAP_DIR = '/var/lib/univention-ldap'
 STATE_DIR = '/var/lib/univention-directory-replication'
 BACKUP_DIR = '/var/univention-backup/replication'
 LDIF_FILE = os.path.join(STATE_DIR, 'failed.ldif')
+SCHEMA_FILE = os.path.join(STATE_DIR, 'schema.conf')
 ROOTPW_FILE = '/etc/ldap/rootpw.conf'
 CURRENT_MODRDN = os.path.join(STATE_DIR, 'current_modrdn')
 MAX_LDAP_RETRIES = int(listener.configRegistry.get('replication/ldap/retries', '30'))
@@ -194,8 +168,7 @@ connection: ldap.ldapobject | None = None
 
 
 def connect(ldif: bool = False) -> ldap.ldapobject:
-    global connection
-    global reconnect
+    global connection, reconnect
 
     if connection and not reconnect:
         return connection
@@ -280,6 +253,14 @@ def subschema_sort(subschema: ldap.schema.subentry.SubSchema, ldap_type: ldap.sc
 
 
 def update_schema(attr: dict[str, list[bytes]]) -> None:
+    tmp = SCHEMA_FILE + '.new'
+    with open(tmp, 'w') as fp:
+        _write_schema(fp, attr)
+
+    os.rename(tmp, SCHEMA_FILE)
+
+
+def _write_schema(fp: IO[str], attr: dict[str, list[bytes]]) -> None:
     def _insert_linebreak(obj: str) -> str:
         # Bug 46743: Ensure lines are not longer than 2000 characters or slapd fails to start
         max_length = 2000
@@ -291,12 +272,6 @@ def update_schema(attr: dict[str, list[bytes]]) -> None:
         obj_lines.append(obj)
         return '\n '.join(obj_lines)
 
-    listener.setuid(0)
-    try:
-        fp = open('/var/lib/univention-ldap/schema.conf.new', 'w')
-    finally:
-        listener.unsetuid()
-
     print('# This schema was automatically replicated from the Primary Directory Node', file=fp)
     print('# Please do not edit this file\n', file=fp)
     subschema = ldap.schema.SubSchema(attr)
@@ -313,17 +288,6 @@ def update_schema(attr: dict[str, list[bytes]]) -> None:
         obj = _insert_linebreak(str(subschema.get_obj(ldap.schema.ObjectClass, oid)))
         print('objectclass %s' % (obj,), file=fp)
 
-    fp.close()
-
-    # move temporary file
-    listener.setuid(0)
-    try:
-        os.rename('/var/lib/univention-ldap/schema.conf.new', '/var/lib/univention-ldap/schema.conf')
-    finally:
-        listener.unsetuid()
-
-    init_slapd('restart')
-
 
 def getOldValues(ldapconn: ldap.ldapobject, dn: str) -> dict[str, list[bytes]]:
     """
@@ -455,7 +419,10 @@ def handler(dn: str, new: dict[str, list[bytes]], listener_old: dict[str, list[b
 
     ud.debug(ud.LISTENER, ud.INFO, 'replication: Running handler %s for: %s' % (operation, dn))
     if dn == 'cn=Subschema':
-        return update_schema(new)
+        with listener.SetUID(0):
+            update_schema(new)
+        init_slapd('restart')
+        return
 
     connect_count = 0
     connected = 0
@@ -680,9 +647,9 @@ def clean() -> Any:
     ud.debug(ud.LISTENER, ud.INFO, 'replication: removing cache')
     # init_slapd('stop')
 
-    # FIXME
+    # FIXME:
     listener.run('/usr/bin/killall', ['killall', '-9', 'slapd'], uid=0)
-    time.sleep(1)  # FIXME
+    time.sleep(1)  # FIXME:
 
     dirname = '/var/lib/univention-ldap/ldap'
     listener.setuid(0)
